(in-package :vigracl.helpers)

(defun copy-array (array)
  "Copies an array into a ewly allocated one -> fast."
  (let ((a (make-array (array-dimensions array) :adjustable nil)))
    (dotimes (i (array-total-size a) a)
      (setf (row-major-aref a i) (row-major-aref array i)))))
(compile 'copy-array)

(defun array-map (func &rest arrays)  
  "Maps a given function to each element of the array and returns a new array"
  (if (null (car arrays))
      '()
    (let* ((new_array   (make-array (array-dimensions (car arrays))
                    :element-type (array-element-type (car arrays))
                    :initial-element 0.0))
	   (width  (array-dimension new_array 0))
           (height (array-dimension new_array 1)))
      (do* ((j 0 (+ j 1)))
	  ((= j height) new_array)
	(do* ((i 0 (+ i 1)))
	    ((= i width))
	  (setf  (aref new_array i j) 
	    (apply func (mapcar #'(lambda(array) (aref array i j)) arrays))))))))
(compile 'array-map)

(defun array-map! (func &rest arrays)  
  "Maps a given function to each element of the array and returns a new array"
  (if (null (car arrays))
      '()
    (let* ((my_array (car arrays))
	   (width  (array-dimension my_array 0))
           (height (array-dimension my_array 1)))
      (do* ((j 0 (+ j 1)))
	  ((= j height) my_array)
	(do* ((i 0 (+ i 1)))
	    ((= i width))
	  (setf  (aref my_array i j) 
	    (apply func (mapcar #'(lambda(array) (aref array i j)) arrays))))))))
(compile 'array-map!)

(defun array-reduce (func array seed)  
  "Reduces an array to one single value as the std. reduce does for lists"
   (let* ((width  (array-dimension array 0))
	  (height (array-dimension array 1))
	  (reduce_var seed))
      (do* ((j 0 (+ j 1)))
	  ((= j height) reduce_var)
	(do* ((i 0 (+ i 1)))
	    ((= i width))
	    (setf reduce_var (funcall func reduce_var  (aref array i j) ))))))
(compile 'array-reduce)


(defun array-for-each-index (func array)  
  "Maps a given function to each pixel coordinate of the array and returns a new array"
   (let* ((width  (array-dimension array 0))
           (height (array-dimension array 1))
           (foo     (do* ((j 0 (+ j 1)))
                         ((= j height) array)
                         (do* ((i 0 (+ i 1)))
                              ((= i width))
			   (funcall func i j)))))))
(compile 'array-for-each-index)