(in-package vigracl.imgproc)

;###############################################################################
;###################         Resize image                   ####################

(ff:def-foreign-call vigra_resizeimage_c 
    ((arr      (:array :float))
     (arr2     (:array :float))
     (width    :int fixnum) 
     (height   :int fixnum)
     (width2    :int fixnum) 
     (height2   :int fixnum)
     (resize_mode :int fixnum)) 
  :returning  (:int fixnum))

(defun resizeimage (arr width2 height2 resize_mode)
  (let* ((width  (array-dimension arr 0))
	 (height (array-dimension arr 1))
	 (arr2 (make-array (list width2 height2)
			   :element-type (array-element-type arr)
			   :initial-element 0.0))
	 (foo   (vigra_resizeimage_c arr arr2 width height width2 height2 resize_mode)))
    (case foo
      ((0) arr2)
      ((1) "Error in vigracl.imgproc:resizeimage: Rotation of image failed!!")
      ((2) "Error in vigracl.imgproc:resizeimage: Resize mode mus be in {0,1,2,3,4}!!"))))


;###############################################################################
;###################         Rotate image                   ####################

(ff:def-foreign-call vigra_rotateimage_c 
    ((arr      (:array :float))
     (arr2     (:array :float))
     (width    :int fixnum) 
     (height   :int fixnum)
     (angle   :float)
     (resize_mode :int fixnum))
  :returning  (:int fixnum))

(defun rotateimage (arr angle resize_mode)
  (let* ((width  (array-dimension arr 0))
	 (height (array-dimension arr 1))
	 (arr2 (make-array (array-dimensions arr)
			   :element-type (array-element-type arr)
			   :initial-element 0.0 ))
	 (foo   (vigra_rotateimage_c arr arr2 width height angle resize_mode)))
    (case foo
      ((0) arr2)
      ((1) "Error in vigracl.imgproc:rotateimage: Rotation of image failed!!")
      ((2) "Error in vigracl.imgproc:rotateimage: Resize mode mus be in {0,1,2,3,4}!!"))))


;###############################################################################
;###################         Affine transform image         ####################

(ff:def-foreign-call vigra_affinewarpimage_c 
    ((arr        (:array :float))
     (arr2       (:array :float))
     (affineMat  (:array :double))
     (width    :int fixnum) 
     (height   :int fixnum)
     (resize_mode :int fixnum))
  :returning  (:int fixnum))


(defun affinewarpimage (arr affinematrix resize_mode)
  (let* ((width  (array-dimension arr 0))
	 (height (array-dimension arr 1))
	 (arr2 (make-array (array-dimensions arr)
			   :element-type (array-element-type arr)
			   :initial-element 0.0 ))
	 (foo   (vigra_affinewarpimage_c arr arr2 affinematrix width height resize_mode)))
    (case foo
      ((0) arr2)
      ((1) "Error in vigracl.imgproc:affinewarpimage: Affine Warp  of image failed!!")
      ((2) "Error in vigracl.imgproc:affinewarpimage: Resize mode mus be in {0,1,2,3,4}!!"))))


;###############################################################################
;###################         Reflect image                  ####################

(ff:def-foreign-call vigra_reflectimage_c 
    ((arr      (:array :float))
     (arr2     (:array :float))
     (width   :int fixnum) 
     (height  :int fixnum)
     (reflect_mode :int fixnum))
  :returning  (:int fixnum))

(defun reflectimage (arr reflect_mode)
  (let* ((width  (array-dimension arr 0))
	 (height (array-dimension arr 1))
	 (arr2 (make-array (array-dimensions arr)
			   :element-type (array-element-type arr)
			   :initial-element 0.0))
	 (foo   (vigra_reflectimage_c arr arr2 width height reflect_mode)))
    (case foo
      ((0) arr2)
      ((1) "Error in vigracl.imgproc:reflectimage: Reflection of image failed!!")
      ((2) "Error in vigracl.imgproc:reflectimage: Reflection mode mus be in {1 (= horizontal), 2 (= vertical), 3 (=both)}!!"))))
	  
	  

;###############################################################################
;###################         Fast Fourier Transform         ####################

(ff:def-foreign-call vigra_fouriertransform_c 
    ((arr      (:array :float))
     (arr2     (:array :float))
     (arr3     (:array :float))
     (width    :int fixnum) 
     (height   :int fixnum))
  :returning  (:int fixnum))

(defun fouriertransform (arr)
  (let* ((width  (array-dimension arr 0))
	 (height (array-dimension arr 1))
	 (arr2 (make-array (list width height)
			   :element-type (array-element-type arr)
			   :initial-element 0.0))
	 (arr3 (make-array (list width height)
			   :element-type (array-element-type arr)
			   :initial-element 0.0))
	 (foo   (vigra_fouriertransform_c arr arr2 arr3  width height)))
    (case foo
      ((0) (cons arr2 arr3))
      ((1) "Error in vigracl.imgproc:fouriertransform: FastFourier Transform of image failed!!"))))
